const url = require('url')
const promisify = require('util.promisify')
const fs = require('fs')
const path = require('path')
const listFiles = promisify(fs.readdir)
const readFile = promisify(fs.readFile)
const writeFile = promisify(fs.writeFile)

const getDir = require('get-dir')
const mustache = require('mustache')
const { mjml2html } = require('mjml')
const fetch = require('node-fetch')

const open = file => readFile(file).then(file => file.toString('utf-8'))
const testMarkup = mjml => /<(?:"[^"]*"['"]*|'[^']*'['"]*|[^'">])+>/.test(mjml)
const isUrl = str => url.parse(str).protocol !== null

function Template(args = {}) {
  Object.assign(this, 
    { mjml: null }, 
    { tags: null }, 
    { partials: null }, 
    args)
}

Template.prototype.load = function() {
  switch(true) {
    case this.mjml === null:
      return listFiles(getDir())
      .then(files => files.find(file => file.indexOf('.mjml') > -1))
      .then(file => open(file))

    case testMarkup(this.mjml):
      return new Promise((resolve, reject) => resolve(this.mjml)) 

    case isUrl(this.mjml):
      return fetch(this.mjml).then(resp => resp.text())

    default:
      var file = this.mjml.indexOf('/') < 0 ? 
        path.join(getDir(), this.mjml) : this.mjml
      return open(file)
  }
}

Template.prototype.render = function() {
  return this.load()
    .then(mjml => mustache.render(mjml, this.tags, this.partials))
    .then(mjml => mjml2html(mjml))
}


module.exports = function(args) {
  var template = new Template(args)
  return template.render()
}